ml.est <- function() {
  ml <- read.table("c:/user/hoehle/work/Java/SIRmcmc/data/ml-estimates.txt",header=T)
  sebetahat  <- ml$betahat/sqrt(ml$m-1)
  segammahat <- ml$gammahat*sqrt(ml$n)
  ml <- cbind(ml,sebetahat,segammahat)

  #mlnozero <- ml[ml$betahat >0,]
  return(ml)
}

trailed <- function(i) {
  if (i<10) return(paste("0",i,sep="")) else return(paste(i)) 
}

########################################################################
# Function to format a real with a specific number of decimal
# positions
#
# x - number to format
# decimals - number of digits behind comma seperator
########################################################################

printformat <- function(x,decimals=4) {
  formatNo <- round(x*10^decimals)/(10^decimals)
  print.default(formatNo,digits=floor(log10(formatNo)+1)+decimals)
}

######################################################################
#
# MCMC estimation of the 10 simulated N=10, a=1 epidemics
# Note: Here we only analyze the datasets generated by the
# SIREpidemics java program.
#
######################################################################

mcmc.est <- function() {
  goodno <<- c(3,4,5,7,8,9,10)
  mcmc <<- list(10)
  beta <- numeric(10)
  gamma <- numeric(10)
  betase <- numeric(10)
  gammase <- numeric(10)

  print("\\hat{beta} = ")
  print("\\hat{gamma} = ")
  print("s.e.(\\hat{beta}) [naive] =")
  print("s.e.(\\hat{beta}) [gamma] =")
  
  for (i in goodno) {
    mcmc[[i]] <<-
      read.table(paste("c:/user/hoehle/work/Java/SIRmcmc/data/MCMC/mcmc-",trailed(i),".txt",sep=""),col.names=c("acc","beta","gamma"))
    
    print(paste("---",i,"---"))
    beta[i] <- print.default(formatC(mean(mcmc[[i]]$beta),digits=4,format="f"))
    gamma[i] <- print.default(formatC(mean(mcmc[[i]]$gamma),digits=4,format="f"))
    betase[i] <- print.default(formatC(sqrt(var(mcmc[[i]]$beta))/sqrt(length(mcmc[[i]]$beta)),digits=4,format="f"))
    gammase[i] <- print.default(formatC(sqrt(var(mcmc[[i]]$gamma))/sqrt(length(mcmc[[i]]$gamma)),digits=4,format="f"))

  }
  
  #beta <<- beta
  #gamma <<- gamma

  #Make the result a data frame and return it.
  res <-
  as.data.frame(cbind(as.numeric(beta),as.numeric(betase),as.numeric(gamma),as.numeric(gammase)))
  names(res) <- c("beta","sebeta","gamma","segamma")

  return(res)
}

######################################################################
#
# Plot the loglik column of a log file
#
#######################################################################

plot.loglik <- function() {
  a <- read.table("C:/user/hoehle/work/Java/SIRmcmc/data/oneill-mcmc-loglik.txt",header=T)
  plot(a$loglik,ylab=expression(log( f(lambda,I)) ),type="l")
}

ps.plot.loglik <- function() {
  hoehlehalfA4plot("C:/user/hoehle/work/tex/SIRmcmc/figs/loglik-oneill-uni-200.eps")
  plot.loglik()
  mydevoff()
}

######################################################################
# Plot beta traj for oneill.
######################################################################


plot.betatrajoneill <- function() {
  a <-
  read.table("C:/user/hoehle/work/Java/SIRmcmc/data/oneill-mcmc-loglik.txt",header=T)
  plot(a$beta,ylab=expression(beta),type="l")
}

ps.plot.betatrajoneill <- function() {
  hoehlehalfA4plot("C:/user/hoehle/work/tex/SIRmcmc/figs/trace-oneill-uni-200.eps")
  plot.betatrajoneill()
  mydevoff()
}

compare.plot <- function() {
  #ml.est()
  #mcmc.est()
  #plot(ml$betahat,beta)

  for (i in 1:10) {
    if (ml$betahat[i] == 0) {
      ml$betahat[i] <<- NA
      ml$gammahat[i] <<- NA
      beta[i]<<- NA
      gamma[i] <<- NA
    }
  }
  
  #plot(ml$gammahat,gamma)
  plot(ml$betahat,beta)
  x <- seq(0,max(beta,ml$betahat),by=0.01)
  
  lines(x,x)
}

######################################################################
#
# Plot X/Y trajectory of the 10 simulated epidemics.
#
######################################################################

plottraj <- function(X = 1) {
  traj <<- list(10)
  maxt <- 0
  legtxt <- numeric(10)
  color <-   rainbow(10)
  plotchars<-c(1,2,3,4,12,6,7,8,9,18)
  linewidth <- c(numeric(4)+1,numeric(3)+2,numeric(3)+3)
  
  for (i in 1:10) {
    traj[[i]] <<-
  read.table(paste("c:/user/hoehle/work/Java/SIRmcmc/data/sim-",trailed(i),".txt",sep=""),col.names=c("t","X","Y","type"))

    #Find the greatest time we have to deal with.
    tmpt <- max(traj[[i]]$t)
    if (tmpt>maxt) {maxt <- tmpt}
  }

  curtraj <- traj[[1]]

  #Chose right values to show
  if (X == 1) {
    curval  <- curtraj$X
    yaxis <- "X(t)"
  } else {
     curval  <- curtraj$Y
     yaxis <- "Y(t)"
  }
  plot(curtraj$t,curval,type="s",xlim=c(0,maxt+3),ylim=c(0,6),xlab="",ylab=yaxis)
  points(max(curtraj$t),curtraj[curtraj$t == max(curtraj$t),]$Y,pch=plotchars[i])
  
  legtxt[1] <- paste(1)
  for (i in 2:10) {
    #Create legend text
    legtxt[i] <- paste(i)
    #Add trajectory to the graph.
    curtraj <- traj[[i]]
    if (X == 1) {
      curval  <- curtraj$X
    } else {
      curval  <- curtraj$Y
    }
    
    points(curtraj$t,curval,type="s",col=color[i],lwd=linewidth[i])
    points(max(curtraj$t),curtraj[curtraj$t == max(curtraj$t),]$Y,pch=plotchars[i])
  }
  #X: legend(14,10,legtxt,pch=plotchars,cex=0.8)
  legend(16.5,6,legtxt,pch=plotchars,cex=0.8)
}

#################################################################
#
# Generate necessary post script figures
# Change plottraj to include $X or $Y
######################################################################

ps.plottraj <- function() {
  hoehlehalfA4plot("c:/user/hoehle/work/tex/SIRmcmc/figs/trajY.eps")
  plottraj()
  mydevoff()
}


boa <- function(fileName) {
  boa.chain.import(fileName, path="C:/user/hoehle/work/Java/SIRmcmcMulti/classes/", type="ASCII")
  boa.print.stats2()
  #boa.plot.geweke(c(fileName),c("beta"),bins=20)
}

problemplot <- function() {
  I <-
  read.table("c:/user/hoehle/work/JAVA/SIRmcmc/classes/i0.txt",col.names=c("hmm","i0","i1","i2","i3","i4"))

  from <- 6600
  to   <- 7000
  plot(b$i0,type="l",ylab="",xlab="",col=3,xlim=c(from,to),ylim=c(min(b$i0[from:to]),0))
  points(b$i1,type="l",col=1)
  legend(from,-20,c(expression(I[1]),expression(I[2])),c(3,1))
  title(paste("I[1] and I[2] from index ",from," to ",to,sep=""))
}

######################################################################
#
# Show acceptance probability as a function of t
#
#
######################################################################
fplot <- function(baseName = "sim-03", movie=0) {
  colors <- rainbow(10)
  f <- read.table(paste("c:/user/hoehle/work/Java/SIRmcmc/data/f-",baseName,".txt",sep=""),col.names=c("i","x","f"),na.strings = "NaN")
  itimes <-
  scan(paste("c:/user/hoehle/work/Java/SIRmcmc/data/",baseName,"-I.txt",sep=""),skip=1)
  rtimes <- scan(paste("c:/user/hoehle/work/Java/SIRmcmc/data/",baseName,"-R.txt",sep=""),skip=2)

  if (movie==1) {
    indexSet <- 1:34

    for (some in indexSet) {
      ones <- numeric(dim(f[f$i==some,])[1])+1
      plot(f$x[f$i==some],pmin(ones,f$f[f$i==some]),type="l",xlab=expression(time),ylab="alpha")
      points(itimes,numeric(length(itimes)),pch="i")
      points(rtimes,numeric(length(itimes)),pch="r")
      points(c(itimes[some+1],itimes[some+1]),c(0,1),col=5,type="l")
      title(paste("I[",some,"]",sep=""))
    }
  } else {
    #Which ones to plot
    indexSet <- c(1,9)
    #Pick first to plot
    some <- indexSet[1]

    #Draw min(1,value), need to generate 1's first.
    ones <- numeric(dim(f[f$i==i,])[1])+1
    #Draw acceptance prob
    plot(f$x[f$i==some],pmin(ones,f$f[f$i==some]),type="l",xlab=expression(t),ylab=expression(alpha(t)))
    #Show I & R events
    points(itimes,numeric(length(itimes)),pch="i")
    points(rtimes,numeric(length(itimes)),pch="r")
    
    #Now use points to draw the rest
    for (i in indexSet[2:length(indexSet)]) {
      points(f[f$i==i,]$x,pmin(ones,f[f$i==i,]$f),type="l",col=i,lty=i)
    }
    legend(5,1,c(expression(I[2]),expression(I[10])),lty=c(1,2))
    #
    #points(f[f$i==i,]$x,pmin(ones,f[f$i==i,]$f),type="l",col=colors[i])
  }
}

#Make ps plot of fplot.
ps.fplot <- function() {
  hoehlefullA4plot("C:/user/hoehle/work/tex/SIRmcmc/figs/fplotI2I10.eps")
  fplot()
  mydevoff()
}


######################################################################
# Shows the trajetory of a epidemic with a single switch point
# Numbers must be in a
# Parameters:
#  s12 - Switch point
######################################################################

plot.traj.regime2 <- function(s12) {
  a <-
  read.table("c:/user/hoehle/work/Java/SIRmcmc/data/switch.txt",col.names=c("t","X","Y","type"))
  
  plot(a$t,a$Y,ylab="No. Of. Infected",type="s",xlab="")
  lines(c(s12,s12),c(0,500),col=3)
}
######################################################################
## Routine to make eps file of plot.traj.regime2
######################################################################
ps.traj.regime2 <- function(s12) {
    hoehlehalfA4plot("c:/user/hoehle/work/tex/SIRmcmc/figs/trajregime.eps")
    plot.traj.regime2(5.1216)
    mydevoff()
}

######################################################################
#
# Show how many infections are in each regime
#
######################################################################

plot.regime.Isizes <- function(N=34) {
  plot(a$m0,type="h",ylim=c(0,34),col=6,ylab="",xlab="")
  lines(c(1,5000),c(34,34))
  text(2500,5,expression(I[1]),cex=2)
  text(2500,28,expression(I[2]),cex=2)
}

######################################################################
# Plot with number of infectious in unit 1 and unit 2, respectively.
######################################################################

plot.2unit.y <- function() {
  #Read in the dataset.  N=(34,34), m=(0,1), beta=(0.1,0.1,0.001), gamma=c(0.3,0.3)
   a <-
     read.table("C:/user/hoehle/work/Java/SIRmcmc/data/2d.txt",col.names=c("t","x1","y1","x2","y2","type"))


  #Find first recovery time, i.e. first eventtime, where the number of
  #infectious has decreased. 
  i <- 1
  while ((a$y1[i] <= a$y1[i+1]) & (i<=length(a$y1[i]))) {
    i <- i+1
  }
  firstRec <- a$t[i]
  a$t <- a$t - firstRec
  print(firstRec)
  #Plot them in the same graph.
  plot(a$t,a$y1,type="s",ylim=c(0,max(a$y2,a$y1)),ylab="No. of Infected",xlab="time")
  points(a$t,a$y2,type="s",lty=3)


   
  legend(10,20,c("Unit 1","Unit 2"),lty=c(1,3),cex=1)
}

plot.2dfinal <- function() {
     a <-
     read.table("C:/user/hoehle/work/Java/SIRmcmc/data/2dfinal.txt",col.names=c("t","Z"))
     hist(breaks=-0.5:max(a$Z+0.5),a$Z,probability = T,xlab="Final Size",main="")

}

######################################################################
# Final size plots of epidemic with N=10, a=1, beta=0.1, gamma=0.1.
######################################################################
plot.10final <- function() {
  hoehlehalfA4plot("c:/user/hoehle/work/tex/SIRmcmc/figs/final10.eps")
  a <-
    read.table("C:/user/hoehle/work/Java/SIRmcmc/data/final10.txt",col.names=c("t","Z"))
  hist(breaks=-0.5:max(a$Z+0.5),a$Z,probability = T,xlab="Final Size",main="")
  points(0:10,f2,col=3,pch=4.,cex=2)
  mydevoff()
}

######################################################################
# Final size of regime switch k=2 beta=0.01, beta=0.1
#
######################################################################

plot.finalregime <- function() {
   a <<-
    read.table("C:/user/hoehle/work/Java/SIRmcmc/data/finalregime.txt",col.names=c("t","regime","Z"))
   n <- 34
   m <- 1

   #Exact sizes of the two involved models
   fm1 <<- fsize.SIR(n,m,0.01*n,Lexp)
   fm2 <<- fsize.SIR(n,m,0.1*n,Lexp)

   #Histogram
   hist(breaks=-0.5:max(a$Z+0.5),a$Z,probability = T,xlab="Final Size",main="",ylim=range(fm1,fm2))

   #Green
   points(0:n,fm1,col=3,pch=4.,cex=2)
   #Red
   points(0:n,fm2,col=2,pch=1.,cex=1.5)

   legend(0,0.8,c("Regime 1","Regime 2"),col=c(3,2),pch=c(4,1))
}

ps.finalregime <- function() {
    hoehlehalfA4plot("c:/user/hoehle/work/tex/SIRmcmc/figs/finalregime.eps")
    plot.finalregime()
    mydevoff()
}

################################################################
# Split 2d sim files to SIREstimate syntax
################################################################

prep2d <- function() {
  data <- read.table("C:/user/hoehle/work/Java/SIRmcmc/data/2d.txt",col.names=c("t","X1","Y1","X2","Y2","type"))

  #Find first recovery and make all times relative to it (zero @).
  data$t <- data$t - min(data[data$type == 0,]$t)
  
  is11 <- numeric(dim(data)[1])
  is11[1] <- 1
  
  for (i in 2:dim(data)[1]) {
    if ((data$X1[i] != data$X1[i-1]) | (data$Y1[i] != data$Y1[i-1])) {
      is11[i] <- 1
    }
  }
  #Augment all events with unit information
  data <- cbind(data,is11)
  #Extract all recoveries.
  recTimes <- data[data$type == 0,]
  
  
  infTimes <- data[data$type == 1,]
  recFile <- cbind(2-recTimes$is11,numeric(length(dim(recTimes)[1]))+1,recTimes$t)
  infFile <- cbind(2-infTimes$is11,numeric(length(dim(infTimes)[1]))+1,infTimes$t)

  #write the stuff to a file
  basePath <- "C:/user/hoehle/work/Java/SIRmcmc/data/"
  recFileName <- paste(basePath,"2d.rec",sep="")
  infFileName <- paste(basePath,"2d.inf",sep="")
  header <- "switch NA\nunit 1 1 24 1 0.1 1 0.3 1\nunit 2 1 25 0 0.1 1 0.3 1\nbetan 0.001 1\ntheta 0"
  write(header,recFileName)
  write(t(recFile), recFileName,ncolumns=3,append=T)
  write(t(infFile), infFileName,ncolumns=3)
  return(is11)
}

#Make a plot of the 2 log likelihoods
csfv.modelselection <- function() {
  M5 <- read.table("C:/user/hoehle/work/Java/SIRmcmc/data/csfv-5regimes-test.txt",header=T)
  M1 <- read.table("C:/user/hoehle/work/Java/SIRmcmc/data/csfv-1regimes-test.txt",header=T)
  M3 <- read.table("C:/user/hoehle/work/Java/SIRmcmc/data/csfv-3regimes-test.txt",header=T)

  min <- min(M5$loglik,M3$loglik,M1$loglik)
  max <- max(M5$loglik,M3$loglik,M1$loglik)
  plot(M5$loglik,type="l",ylim=c(min,max),ylab="loglik")
  points(M3$loglik,type="l",col=2,lty=2)
  points(M1$loglik,type="l",col=3,lty=3)
  legend(100,min+8/9*(max-min),c("M5","M3","M1"),lty=c(1,2,3),col=c(1,2,3))
}

######################################################################
#
# Independent sampling, i.e. just use order constraint
# This function creates a plot for beta and gamma and show the
# relevant stats of the sampling distributions.
######################################################################

plot.indep.densities <- function() {
 a <- read.table("c:/user/hoehle/work/Java/SIRmcmc/data/oneill-independent-samples.txt",header=T)
 #Do the beta density plot
 hoehlehalfA4plot("c:/user/hoehle/work/tex/SIRmcmc/figs/beta-independent.eps")
 plot(density(a$beta110,kernel="gaussian"),main="",xlab="")
 mydevoff()
 #Do the gamma density plot
 hoehlehalfA4plot("c:/user/hoehle/work/tex/SIRmcmc/figs/gamma-independent.eps")
 plot(density(a$gamma110,kernel="gaussian"),main="",xlab="")
 mydevoff()

 #Show mean and variance
 print("beta")
 print(mean(a$beta110))
 print(var(a$beta110))
 print("gamma")
 print(mean(a$gamma110))
 print(var(a$gamma110))
 

}

######################################################################
#
# Show how log likelihood varies as a function of I_0 in the Oneill
# example
######################################################################

I0LikEffect <- function() {
  #Read the data which contains 100 vals for each I0 value between
  #-101 and -1 with a step of 5
  a <- read.table("C:/user/hoehle/work/Java/SIRmcmc/data/I0-oneill.txt")
  a <- as.matrix(a[,2:101])

  #Now do a boxplot containing each trial
  name <- list()
  for (i in 1:21) {
    d[[i]] <- as.vector(c[i,])
    name[[i]] <- -101 + 5*(i-1)
  }
  boxplot(d,names=name,xlab=expression(I[0]),ylab=expression(log(f(tau,I))))

}

plot.I0LikEffect <- function() {
  hoehlefullA4plot("C:/user/hoehle/work/tex/SIRmcmc/figs/I0LikEffect-oneill.eps")
  I0LikEffect()
  mydevoff()
}

mml.mean <- function(tau,theta) {
  for (i in 0:(length(tau)-1)) {
    if (i == 0) {
      res <- -0.5*theta
    } else {
      res <- append(res,0.5*(res[i]+tau[i]))
    }
  }
  return(res)
}

log.anal <- function(meanvar=T,burnin=0,filter=1) {
  a<<-read.table("C:/user/hoehle/work/Java/LadyBug/classes/log.txt",header=T)
  #End
  e <- length(a[,1])
  #Start
  s <- 1+burnin
  sampleno <- seq(from=s,to=e,by=filter)
  
  plot(a$beta111[sampleno],type="l")
  plot(a$gamma111[sampleno],type="l")
  plot(a$beta211[sampleno],type="l")
  plot(a$gamma211[sampleno],type="l")
  plot(a$betaN1[sampleno],type="l")
  #if (!meanvar) {
  #  plot(a$gamma[sampleno]/a$delta[sampleno],type="l",ylab="mean(I)")
  #  plot(sqrt(a$gamma[sampleno])/a$delta[sampleno],type="l",ylab="s.dev(I)")
  #}
  plot(a$I0[sampleno],type="l")
  print("beta")
  print(mean(a$beta111[sampleno]))
  print("gamma")
  print(mean(a$gamma[sampleno]))
  print("delta")
  print(mean(a$delta[sampleno]))
  if (!meanvar) {
    print("delta/gamma")
    print(mean(a$gamma[sampleno]/a$delta[sampleno]))
  }
}


######################################################################
# ESS function
######################################################################

doIt <- function() {
  source("C:/user/hoehle/work/Java/LadyBug/data/R/analyze.S")
  source("c:/Programmer/R/boa/boa.R")
  source("c:/Programmer/R/boa/boa-hoehle.R")
  boa.start()
  #I <- read.table("c:/user/hoehle/work/Java/SIRmcmc/data/oneill-I-times.txt")
  #tau <- read.table("c:/user/hoehle/work/Java/SIRmcmc/data/oneill.rec",skip=4)[,3]
  
}













